import processing.opengl.*;
import treemap.*;
import peasy.*;


Treemap map;
PeasyCam cam;

String[] DeweyClasses;
String[] MonthNames;

TreeMap Deweys;

int X = 1000;
int Y = 700;

int monthCounter = 0;
int mode = 0;
boolean grid = false;

int[] _minCheckout= new int[12];
int[] _maxCheckout= new int[12];
int[] _min= new int[12];
int[] _max= new int[12];

 
float xRotation = 0;
float yRotation = 0;
float zRotation = 0;
float xRotationTarget = 0;
float yRotationTarget = 0;
float zRotationTarget = 0;

void setup( ) {
  
  //Initialize Frame with OPEN GGL
  size(X, Y, OPENGL);
  colorMode(HSB,360,100,100,100);
  frameRate(30);
  smooth();
  initialize();
  loadData();
  
  //Initialize Camera
  cam = new PeasyCam(this, 100, 0,-200, 1200);
  //cam = new PeasyCam(this, 500);
  cam.setMinimumDistance(5);
  cam.setMaximumDistance(3000);
  
  strokeWeight(0.25f);
  
  PFont font = createFont("MyriadPro-Regular", 13);
  textFont(font);
  
  
  /*
  String[] lines = loadStrings("equator.txt");
  for (int i = 0; i < lines.length; i++) {
    mapData.addWord(lines[i], 0);
  }
  */
   
  
  
}



void draw( ) {
  background(165,0,50,100);
  if(grid)
    draw3dGrid();
    
  drawAxis();
  //println("month counter is "+ monthCounter);
  WordMap mapData = new WordMap( );
  
  for(Object key : Deweys.keySet()){
    DeweyCategory d = (DeweyCategory) Deweys.get(key);
    //println("Inserting "+key);
    for(int i = 0; i <= d.avgCkoutDuration[monthCounter]; i++)
      mapData.addWord(d.deweyName, d.deweyCode);
  }
  
  mapData.finishAdd( );
  map = new Treemap(mapData, 0, 0, width*1.5, height);
  map.setLayout(new SliceLayout(1));
  map.updateLayout();
  map.draw( );  
  
  cam.beginHUD();
  textAlign(CENTER, CENTER);
  PFont font = createFont("MyriadPro-Regular", 25);
  textFont(font);

  fill(0,10,100,100);
  textAlign(LEFT, CENTER);
  text("Three Dimensional TreeMap", 5, 20);
  
  font = createFont("MyriadPro-Regular", 12);
  textFont(font);
  textAlign(RIGHT, CENTER);
  text("W 2012 - MAT259", X-5, 10);
  text("Ankit Srivastava", X-5, 25);
  
  font = createFont("MyriadPro-Regular", 40);
  textFont(font);

  textAlign(RIGHT, BOTTOM);
  text(MonthNames[monthCounter], X-5, Y);

  font = createFont("MyriadPro-Regular", 15);
  textFont(font);
  textAlign(LEFT, BOTTOM);
  text("X-Y Axis - Avg Checkout Duration/Month", 5, Y-40);
  text("Z Axis - Number of Checkouts/Month", 5, Y-20);
  text("X-Z Axis - Number of Checkouts/Day", 5, Y);
  
  cam.endHUD();
  
  

}

void draw3dGrid(){
  pushMatrix();
  stroke(0, 100,100,100);
  line(-X,0,0, X, 0, 0);
  stroke(120, 100,100,100);
  line(0,-Y,0, 0, Y, 0);
  stroke(240, 100,100,100);
  line(0,0,-1000, 0, 0, 1000);
  popMatrix();
}

void drawAxis(){
  strokeWeight(1);
  stroke(0, 100,0,100);
  line(-100,-100,-100, 100, -100, -100);
  line(-100,-100,-100, -100, 100, -100);
  line(-100,-100,-100, -100, -100, 100);
  
  PFont font = createFont("MyriadPro-Regular", 25);
  textFont(font);
  textAlign(RIGHT, BOTTOM);
  text("X AXIS", 100, -100, -100);
  pushMatrix();
  rotateZ(-PI/2);
  textAlign(LEFT, BOTTOM);
  text("Y AXIS", -100, -100, -100);
  popMatrix();
  pushMatrix();
  rotateY(-PI/2);
  text("Z AXIS", -100, -100, 100);
  popMatrix();
}

void keyPressed() {
  if (key == 'r') {
    cam.reset(5000);
  } 

  if (key == 'a') {
    cam.rotateY(PI/2.0);
    cam.lookAt(random(-100.0,50.0),random(-200.0,100.0),random(-100.0,300.0), 1000);
    xRotationTarget = random(10.0);
    yRotationTarget = random(1.0);
    zRotationTarget = random(1.0);
  }
  
  if(key == 'q') {
    monthCounter = monthCounter+1;
    if(monthCounter > 11)
      monthCounter = 0;
  }
  if(key == 'w') {
    monthCounter = monthCounter - 1;
    if(monthCounter < 0)
      monthCounter = 0;  
  }
  
  if(key =='m'){
    mode = mode + 1;
    if(mode > 2)
      mode = 0;
  }
  
  if(key == 'g')
    grid = !grid;  
}

void mouseReleased(){
    float[] rotations = cam.getRotations();
    
    xRotation = rotations[0];
    yRotation = rotations[1];
    zRotation = rotations[2];
    xRotationTarget = rotations[0];
    yRotationTarget = rotations[1];
    zRotationTarget = rotations[2];    
}

void initialize(){
  DeweyClasses = new String[]{"Computer Science, Information & General Works","Philosophy & Psychology","Religion","Social Sciences","Language","Science","Technology & Applied Science","Arts & Recreation","Literature","History & Geography"};
  MonthNames = new String[]{"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
  Deweys = new TreeMap();
  for(int i = 0; i < 1000; i=i+100){
    DeweyCategory dewey = new DeweyCategory(i);
    Deweys.put(i, dewey);
  }
  this._maxCheckout = new int[12];
  this._minCheckout = new int[12];
  for(int i = 0; i < 12; i++){
    this._maxCheckout[i] = 0;
    this._minCheckout[i] = 99999;
    this._min[i] = 99999;
    this._max[i] = 0;
  }  
}

void loadData(){
  String[] lines = loadStrings("count_per_day_2011.txt");  
  
  //Line Elements.
  for (int i = 0; i < lines.length; i++) {
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    //Initialize Dewey Data to temp variables.
    int subdewey = (Integer.parseInt(data[1])%100)/10;
    int _month = Integer.parseInt(data[2])-1;
    int _day = Integer.parseInt(data[3])-1;
    int value = Integer.parseInt(data[4]);
    //println(subdewey+ " "+_month+ " "+_day+ " "+value);
    //Store Dewey data.
    d.putDataElement(subdewey, _month, _day, value);           
  }
  
  //Header Information
  lines = loadStrings("count_per_month_2011.txt");
  for(int i = 0; i < lines.length; i++){
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    int _month = Integer.parseInt(data[1]);
    int num = Integer.parseInt(data[2]);
    float avg = Float.parseFloat(data[3]);
    d.putData(_month-1, num, avg);
  }  
}



