import processing.core.*; 
import processing.xml.*; 

import processing.opengl.*; 
import treemap.*; 
import peasy.*; 
import java.util.Random; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class ThreeDTreeMap extends PApplet {






Treemap map;
PeasyCam cam;

String[] DeweyClasses;
String[] MonthNames;

HashMap Deweys;

int X = 1000;
int Y = 700;
int monthCounter = 0;

float xRotation = 0;
float yRotation = 0;
float zRotation = 0;
float xRotationTarget = 0;
float yRotationTarget = 0;
float zRotationTarget = 0;

public void setup( ) {
  
  //Initialize Frame with OPEN GGL
  size(X, Y, OPENGL);
  colorMode(HSB,360,100,100,100);
  frameRate(30);
  smooth();
  initialize();
  loadData();
  
  //Initialize Camera
  //cam = new PeasyCam(this, 100, 0,-200, 1200);
  cam = new PeasyCam(this, 500);
  cam.setMinimumDistance(10);
  cam.setMaximumDistance(3000);
  
  strokeWeight(0.25f);
  
  PFont font = createFont("Serif", 13);
  textFont(font);
  
  WordMap mapData = new WordMap( );
  String[] lines = loadStrings("equator.txt");
  
  for (int i = 0; i < lines.length; i++) {
    mapData.addWord(lines[i]);
  }
  
  mapData.finishAdd( );
  map = new Treemap(mapData, 0, 0, width, height);
}



public void draw( ) {
  background(0);
  draw3dGrid();
  map.setLayout(new SliceLayout(1));
  map.updateLayout();
  map.draw( );  
}

public void draw3dGrid(){
  pushMatrix();
  stroke(0, 100,100,100);
  line(-X,0,0, X, 0, 0);
  stroke(120, 100,100,100);
  line(0,-Y,0, 0, Y, 0);
  stroke(240, 100,100,100);
  line(0,0,-1000, 0, 0, 1000);
  popMatrix();
}

public void keyPressed() {
  if (key == 'r') {
    cam.reset(5000);
  } 

  if (key == 'a') {
    cam.rotateY(PI/2.0f);
    cam.lookAt(random(-100.0f,50.0f),random(-200.0f,100.0f),random(-100.0f,300.0f), 1000);
    xRotationTarget = random(10.0f);
    yRotationTarget = random(1.0f);
    zRotationTarget = random(1.0f);
  }
  
  if(key == 'q') {
    monthCounter = monthCounter+1;
    if(monthCounter > 11)
      monthCounter = 0;
  }
  if(key == 'w') {
    monthCounter = monthCounter - 1;
    if(monthCounter < 0)
      monthCounter = 0;  
  }
}

public void mouseReleased(){
    float[] rotations = cam.getRotations();
    
    xRotation = rotations[0];
    yRotation = rotations[1];
    zRotation = rotations[2];
    xRotationTarget = rotations[0];
    yRotationTarget = rotations[1];
    zRotationTarget = rotations[2];    
}

public void initialize(){
  Deweys = new HashMap();
  for(int i = 0; i < 1000; i=i+100){
    DeweyCategory dewey = new DeweyCategory(i);
    Deweys.put(i, dewey);
  }
  DeweyClasses = new String[]{"Computer Science, Information & General Works","Philosophy & Psychology","Religion","Social Sciences","Language","Science","Technology & Applied Science","Arts & Recreation","Literature","History & Geography"};
  MonthNames = new String[]{"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
}

public void loadData(){
  String[] lines = loadStrings("count_per_day_2011.txt");  
  
  //Line Elements.
  for (int i = 0; i < lines.length; i++) {
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    //Initialize Dewey Data to temp variables.
    int subdewey = (Integer.parseInt(data[1])%100)/10;
    int _month = Integer.parseInt(data[2])-1;
    int _day = Integer.parseInt(data[3])-1;
    int value = Integer.parseInt(data[4]);
    //println(subdewey+ " "+_month+ " "+_day+ " "+value);
    //Store Dewey data.
    d.putDataElement(subdewey, _month, _day, value);           
  }
  
  //Header Information
  lines = loadStrings("count_per_month_2011.txt");
  for(int i = 0; i < lines.length; i++){
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    int _month = Integer.parseInt(data[1]);
    int num = Integer.parseInt(data[2]);
    float avg = Float.parseFloat(data[3]);
    d.putData(_month-1, num, avg);
  }  
}



class DeweyCategory{
 String deweyName;
 int deweyCode;
 
 float[] avgCkoutDuration;
 int[] numberCheckouts;
 
 //sub_dewey>month-> day
 int[][][] DataArray;
 
 int _max;
 int _min;
 
 public DeweyCategory(int deweyCode){
   this.deweyCode = deweyCode;
   this.deweyName = getDeweyName(deweyCode);
   this.avgCkoutDuration = new float[12];
   this.numberCheckouts = new int[12];
   
   this.DataArray = new int[10][12][31];
   
   for(int i = 0; i < 10 ; i++)
     for(int j = 0; j < 12 ; j++)
       for(int k = 0; k < 31 ; k++)
         this.DataArray[i][j][k] = 0;
         
   this._max = 0;
   this._min = 999999;
 }

 public String getDeweyName(int deweyCode){   
   println("DeweyCode = "+ deweyCode);
   return DeweyClasses[deweyCode/100];
 }
 
 public void putDataElement(int sub_dewey, int _month, int _day, int value){
   this.DataArray[sub_dewey][_month][_day] = value;
   
   if(value> this._max)
     this._max = value;
     
   if(value < this._min)
     this._min = value;      
 } 
 
 public int getDataElement(int sub_dewey, int _month, int _day){
   return this.DataArray[sub_dewey][_month][_day];
 }
 
 public void putData(int _month, int _numCkout, float _avgDuration){
   this.avgCkoutDuration[_month] = _avgDuration;
   this.numberCheckouts[_month] = _numCkout;
 } 
}
/*

Query to get Day wise data for all months, deweys and sub-deweys

select FLOOR(deweyClass/100)*100 dewey, FLOOR(deweyClass/10)*10 deweySub, month(cout) month, day(cout) day, count(*)
from spl0.inraw
where
(deweyClass <> 'null' OR deweyClass <> NULL)
and year(cout) = 2011
group by dewey, deweySub, month, day 
order by dewey, deweySub, month, day;






























*/
class WordItem extends SimpleMapItem {
  int z;
  String word;
  float d;
  Random random1;
  WordItem(String word) {
    this.word = word;
    z = 0;
    d = textWidth(this.word)*10;
    random1 = new Random();
  }
  public void draw( ) {
    pushMatrix();
    noFill();
    stroke(0,0,100,100);
    translate(x,y,0);
    translate(0,0,d/2); //added by josh.  You want everything to start on the same z-axis line so your first translation needs to have a z-axis value that's proportional to your box's overall height.   
    box(w,h,d);
    
    pushMatrix();
    translate(0,0,-d*0.55f); //added by josh.  I'm not sure why it's -5.5 and not just -5... but it seems to line up now.  
    for(int i = 0; i<10; i++){
    
      if(i%2==0){
        stroke(0,100,100,100);
        //fill(0,100,100,2);
      }
      else{
        stroke(120,100,100,100);
        //fill(120,100,100,2);
      }
      
      translate(0,0,d/10);
      box(w,h,d/10);
     
      pushMatrix();
      translate(-w*0.515f,0,0);
      for(int j=0; j<31; j++){
        /*
        if(j%2==0){
          stroke(180,100,100,100);
          fill(180,100,100,100);
        }
        else{
          stroke(240,100,100,100);
          fill(240,100,100,100);
        }*/
        
        fill(0,100, random1.nextInt(100), 100);
        translate(w/31,0,0);
        box(w/31, h, d/10);
      }
      popMatrix();
      
    }
    popMatrix();
      
    if (w > textWidth(word) + 6) {
      if (h > textAscent( ) + 6) {
        textAlign(CENTER, CENTER);
        text(word, 0, 0);
      }
    }
    popMatrix();

    fill(255);
    rectMode(CORNER);
    //rect(x, y, w, h);    
    fill(255);
    

  }
}

class WordMap extends SimpleMapModel {
  HashMap words;
  WordMap( ) {
    words = new HashMap( );
  }
  public void addWord(String word) {
    WordItem item = (WordItem) words.get(word);
    if (item == null) {
      item = new WordItem(word);
      words.put(word, item);
    }
    item.incrementSize( );
  }
  public void finishAdd( ) {
    items = new WordItem[words.size( )];
    words.values( ).toArray(items);
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "ThreeDTreeMap" });
  }
}
