import processing.opengl.*;
import treemap.*;
import peasy.*;
import java.util.Random;

Treemap map;
PeasyCam cam;

String[] DeweyClasses;
String[] MonthNames;

HashMap Deweys;

int X = 1000;
int Y = 700;
int monthCounter = 0;

float xRotation = 0;
float yRotation = 0;
float zRotation = 0;
float xRotationTarget = 0;
float yRotationTarget = 0;
float zRotationTarget = 0;

void setup( ) {
  
  //Initialize Frame with OPEN GGL
  size(X, Y, OPENGL);
  colorMode(HSB,360,100,100,100);
  frameRate(30);
  smooth();
  initialize();
  loadData();
  
  //Initialize Camera
  //cam = new PeasyCam(this, 100, 0,-200, 1200);
  cam = new PeasyCam(this, 500);
  cam.setMinimumDistance(10);
  cam.setMaximumDistance(3000);
  
  strokeWeight(0.25f);
  
  PFont font = createFont("Serif", 13);
  textFont(font);
  
  WordMap mapData = new WordMap( );
  String[] lines = loadStrings("equator.txt");
  
  for (int i = 0; i < lines.length; i++) {
    mapData.addWord(lines[i]);
  }
  
  mapData.finishAdd( );
  map = new Treemap(mapData, 0, 0, width, height);
}



void draw( ) {
  background(0);
  draw3dGrid();
  map.setLayout(new SliceLayout(1));
  map.updateLayout();
  map.draw( );  
}

void draw3dGrid(){
  pushMatrix();
  stroke(0, 100,100,100);
  line(-X,0,0, X, 0, 0);
  stroke(120, 100,100,100);
  line(0,-Y,0, 0, Y, 0);
  stroke(240, 100,100,100);
  line(0,0,-1000, 0, 0, 1000);
  popMatrix();
}

void keyPressed() {
  if (key == 'r') {
    cam.reset(5000);
  } 

  if (key == 'a') {
    cam.rotateY(PI/2.0);
    cam.lookAt(random(-100.0,50.0),random(-200.0,100.0),random(-100.0,300.0), 1000);
    xRotationTarget = random(10.0);
    yRotationTarget = random(1.0);
    zRotationTarget = random(1.0);
  }
  
  if(key == 'q') {
    monthCounter = monthCounter+1;
    if(monthCounter > 11)
      monthCounter = 0;
  }
  if(key == 'w') {
    monthCounter = monthCounter - 1;
    if(monthCounter < 0)
      monthCounter = 0;  
  }
}

void mouseReleased(){
    float[] rotations = cam.getRotations();
    
    xRotation = rotations[0];
    yRotation = rotations[1];
    zRotation = rotations[2];
    xRotationTarget = rotations[0];
    yRotationTarget = rotations[1];
    zRotationTarget = rotations[2];    
}

void initialize(){
  Deweys = new HashMap();
  for(int i = 0; i < 1000; i=i+100){
    DeweyCategory dewey = new DeweyCategory(i);
    Deweys.put(i, dewey);
  }
  DeweyClasses = new String[]{"Computer Science, Information & General Works","Philosophy & Psychology","Religion","Social Sciences","Language","Science","Technology & Applied Science","Arts & Recreation","Literature","History & Geography"};
  MonthNames = new String[]{"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
}

void loadData(){
  String[] lines = loadStrings("count_per_day_2011.txt");  
  
  //Line Elements.
  for (int i = 0; i < lines.length; i++) {
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    //Initialize Dewey Data to temp variables.
    int subdewey = (Integer.parseInt(data[1])%100)/10;
    int _month = Integer.parseInt(data[2])-1;
    int _day = Integer.parseInt(data[3])-1;
    int value = Integer.parseInt(data[4]);
    //println(subdewey+ " "+_month+ " "+_day+ " "+value);
    //Store Dewey data.
    d.putDataElement(subdewey, _month, _day, value);           
  }
  
  //Header Information
  lines = loadStrings("count_per_month_2011.txt");
  for(int i = 0; i < lines.length; i++){
    String[] data = lines[i].split(",");
    DeweyCategory d = (DeweyCategory) Deweys.get(Integer.parseInt(data[0]));
    int _month = Integer.parseInt(data[1]);
    int num = Integer.parseInt(data[2]);
    float avg = Float.parseFloat(data[3]);
    d.putData(_month-1, num, avg);
  }  
}



